@@ -43,9 +43,12 @@ enum
 
 enum
 {
+  SIGNAL_POPUP_SHOW,
   POPUP_N_SIGNALS
 };
 
+static gint signals[POPUP_N_SIGNALS];
+
 static GObject *hildon_desktop_popup_window_constructor (GType gtype, 
                                                          guint n_params, 
                                                          GObjectConstructParam *params);
@@ -60,6 +63,7 @@ static void hildon_desktop_popup_window_set_property (GObject *object,
                                                       const GValue *value, 
                                                       GParamSpec *pspec);
 
+static void hildon_desktop_popup_finalize (GObject *object);
 static void hildon_desktop_popup_window_realize (GtkWidget *widget);
 static void hildon_desktop_popup_window_unrealize (GtkWidget *widget);
 static void hildon_desktop_popup_window_show (GtkWidget *widget);
@@ -67,7 +71,7 @@ static void hildon_desktop_popup_window_hide (GtkWidget *widget);
 static void hildon_desktop_popup_window_show_all (GtkWidget *widget);
 static void hildon_desktop_popup_window_hide_all (GtkWidget *widget);
 
-static gboolean hildon_desktop_popup_window_enter_notify (GtkWidget *widget, GdkEventCrossing *event);
+static gboolean hildon_desktop_popup_window_motion_notify (GtkWidget *widget, GdkEventMotion *event);
 static gboolean hildon_desktop_popup_window_leave_notify (GtkWidget *widget, GdkEventCrossing *event);
 
 static gboolean hildon_desktop_popup_window_visibility_notify (GtkWidget          *widget,
@@ -78,6 +82,12 @@ static gboolean hildon_desktop_popup_window_composited_leave_notify (GtkWidget *
 						     		     GdkEventCrossing *event,
 						     		     HildonDesktopPopupWindow *popup);
 
+static gboolean hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
+		                                                       GdkEventButton *event,
+	                                                               HildonDesktopPopupWindow *popup);
+
+static gboolean popup_grab_on_window (GdkWindow *window, guint32 activate_time, gboolean grab_keyboard);
+
 static gboolean
 hildon_desktop_popup_window_button_release_event (GtkWidget *widget, GdkEventButton *event);
 
@@ -92,6 +102,7 @@ struct _HildonDesktopPopupWindowPrivate
   gpointer			    position_func_data;
 
   gboolean 			    have_xgrab;
+  GtkWidget 			   *pane_with_grab;
 
   GtkWidget			   *attached_widget;
 };
@@ -104,8 +115,9 @@ hildon_desktop_popup_window_init (HildonDesktopPopupWindow *popup)
   popup->priv->extra_panes   = NULL;
   popup->priv->n_extra_panes = 0;
 
-  popup->priv->have_xgrab = FALSE;
+  popup->priv->have_xgrab = FALSE; 
 
+  popup->priv->pane_with_grab  =
   popup->priv->attached_widget = NULL;
 }
 
@@ -118,8 +130,9 @@ hildon_desktop_popup_window_class_init (HildonDesktopPopupWindowClass *popup_cla
   object_class->constructor  = hildon_desktop_popup_window_constructor;
   object_class->set_property = hildon_desktop_popup_window_set_property;
   object_class->get_property = hildon_desktop_popup_window_get_property;
+  object_class->finalize     = hildon_desktop_popup_finalize;
 
-  widget_class->enter_notify_event      = hildon_desktop_popup_window_enter_notify;
+  widget_class->motion_notify_event     = hildon_desktop_popup_window_motion_notify;
   widget_class->leave_notify_event      = hildon_desktop_popup_window_leave_notify;
   widget_class->button_release_event    = hildon_desktop_popup_window_button_release_event;
 
@@ -133,6 +146,15 @@ hildon_desktop_popup_window_class_init (HildonDesktopPopupWindowClass *popup_cla
   
   g_type_class_add_private (object_class, sizeof (HildonDesktopPopupWindowPrivate));
 
+  signals[SIGNAL_POPUP_SHOW] =
+        g_signal_new ("popup-window",
+                      G_OBJECT_CLASS_TYPE (object_class),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (HildonDesktopPopupWindowClass,popup_window),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
   g_object_class_install_property (object_class,
                                    PROP_POPUP_N_PANES,
                                    g_param_spec_uint(
@@ -199,9 +221,17 @@ hildon_desktop_popup_window_constructor (GType gtype,
   {	  
     popup->priv->extra_panes[i] = gtk_window_new (GTK_WINDOW_POPUP);
 
+    g_object_ref (G_OBJECT (popup->priv->extra_panes[i]));
+    gtk_object_sink (GTK_OBJECT (popup->priv->extra_panes[i]));		 
+
     gtk_window_set_type_hint (GTK_WINDOW (popup->priv->extra_panes[i]),
 		    	      GDK_WINDOW_TYPE_HINT_MENU);
 
+    g_signal_connect (popup->priv->extra_panes[i],
+		      "button-release-event",
+		      G_CALLBACK (hildon_desktop_popup_window_composited_button_release),
+		      (gpointer)popup); 
+    
     g_signal_connect (popup->priv->extra_panes[i],
 		      "leave-notify-event",
 		      G_CALLBACK (hildon_desktop_popup_window_composited_leave_notify),
@@ -215,6 +245,19 @@ hildon_desktop_popup_window_constructor (GType gtype,
   return object;
 }
 
+static void 
+hildon_desktop_popup_finalize (GObject *object)
+{
+  HildonDesktopPopupWindow *popup = HILDON_DESKTOP_POPUP_WINDOW (object);
+  register gint i;
+  
+  if (popup->priv->extra_panes)
+   for (i=0; i < popup->priv->n_extra_panes; i++)
+     g_object_unref (popup->priv->extra_panes[i]);
+
+  G_OBJECT_CLASS (hildon_desktop_popup_window_parent_class)->finalize (object);
+}	
+
 /* At some point I will use macros :) */
 
 static void 
@@ -289,19 +332,49 @@ hildon_desktop_popup_window_hide_all (GtkWidget *widget)
     gtk_widget_hide_all (popup->priv->extra_panes[i]);
 }
 
-static gboolean
-hildon_desktop_popup_window_enter_notify (GtkWidget        *widget,
-		                          GdkEventCrossing *event)
+static gboolean 
+hildon_desktop_popup_window_motion_notify (GtkWidget      *widget,
+                                           GdkEventMotion *event)
 {
   HildonDesktopPopupWindow *popup = HILDON_DESKTOP_POPUP_WINDOW (widget);
+  gboolean in_panes_area = FALSE;
+  gint w,h,x,y,i;
 
-  g_debug ("%s enter_notify %p",__FILE__,popup);
-  /* We have to grab the pointer in here, we should get this when we come from
-   * a composited window
-   */ 
+  w = widget->allocation.width;
+  h = widget->allocation.height;
+
+  gtk_widget_get_pointer (widget, &x, &y);
+
+  if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+    in_panes_area = TRUE;
+
+  if (!in_panes_area)
+  {	  
+    for (i=0; i < popup->priv->n_extra_panes; i++)
+    {
+      w = popup->priv->extra_panes[i]->allocation.width;
+      h = popup->priv->extra_panes[i]->allocation.height;
+
+      gtk_widget_get_pointer (popup->priv->extra_panes[i], &x, &y);
+
+      if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+      {	       
+        gtk_grab_remove (widget);		 
+	 
+        popup_grab_on_window
+         (popup->priv->extra_panes[i]->window, GDK_CURRENT_TIME, TRUE);
+
+        gtk_grab_add (popup->priv->extra_panes[i]);
+
+        popup->priv->pane_with_grab = popup->priv->extra_panes[i];
+	popup->priv->have_xgrab = FALSE;
+	
+	break;
+      }
+    }
+  }
   
-  return 
-    GTK_WIDGET_CLASS (hildon_desktop_popup_window_parent_class)->enter_notify_event (widget, event);	
+  return TRUE; 
 }
 
 static gboolean 
@@ -309,22 +382,97 @@ hildon_desktop_popup_window_leave_notify (GtkWidget        *widget,
 					  GdkEventCrossing *event)
 {
   HildonDesktopPopupWindow *popup = HILDON_DESKTOP_POPUP_WINDOW (widget);
+  gint w,h,x,y,i;
+  gboolean in_panes_area = FALSE;
 
  /* We have to ungrab the pointer in here, we should get this when we go to
   * a composited window
   */
-  g_debug ("%s leave_notify %p",__FILE__,popup);
+
+  for (i=0; i < popup->priv->n_extra_panes; i++)
+  {
+    w = popup->priv->extra_panes[i]->allocation.width;
+    h = popup->priv->extra_panes[i]->allocation.height;
+
+    gtk_widget_get_pointer (popup->priv->extra_panes[i], &x, &y);
+
+    if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+    {	    
+      in_panes_area = TRUE;
+      break;
+    }
+  }
+
+  if (in_panes_area)
+  {	  
+    gtk_grab_remove (GTK_WIDGET (popup));
+ 
+    popup_grab_on_window 
+      (popup->priv->extra_panes[i]->window, GDK_CURRENT_TIME, TRUE);
+
+    gtk_grab_add (popup->priv->extra_panes[i]);
+
+    popup->priv->pane_with_grab = popup->priv->extra_panes[i];
+    popup->priv->have_xgrab = FALSE;
+  }
   
   return 
     GTK_WIDGET_CLASS (hildon_desktop_popup_window_parent_class)->leave_notify_event (widget, event);	
 }
 
-static gboolean 
+static gboolean
 hildon_desktop_popup_window_composited_leave_notify (GtkWidget *widget,
-						     GdkEventCrossing *event,
-						     HildonDesktopPopupWindow *popup)
+                                                     GdkEventCrossing *event,
+                                                     HildonDesktopPopupWindow *popup)
 {
+  gint w,h,x,y,i;
+  gboolean in_panes_area = FALSE;
+  
+  for (i=0; i < popup->priv->n_extra_panes; i++)
+  {
+    w = popup->priv->extra_panes[i]->allocation.width;
+    h = popup->priv->extra_panes[i]->allocation.height;
+    
+    gtk_widget_get_pointer (popup->priv->extra_panes[i], &x, &y);
+       
+    if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+    {	    
+      in_panes_area = TRUE;
+      break;
+    }
+  }
+  
+  if (in_panes_area)
+  {
+    gtk_grab_remove (widget);
 
+    popup_grab_on_window
+      (popup->priv->extra_panes[i]->window, GDK_CURRENT_TIME, TRUE);
+
+    gtk_grab_add (popup->priv->extra_panes[i]);
+
+    popup->priv->pane_with_grab = popup->priv->extra_panes[i];
+    popup->priv->have_xgrab = FALSE;
+  }
+  else
+  {
+    gtk_widget_get_pointer (GTK_WIDGET (popup), &x, &y);
+
+    w = GTK_WIDGET (popup)->allocation.width;
+    h = GTK_WIDGET (popup)->allocation.height;
+
+    if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+    {
+     gtk_grab_remove (widget);
+
+     popup_grab_on_window (GTK_WIDGET (popup)->window, GDK_CURRENT_TIME, TRUE);
+
+     gtk_grab_add (GTK_WIDGET (popup));
+
+     popup->priv->pane_with_grab = NULL;
+     popup->priv->have_xgrab = TRUE;
+    }
+  }
 
   return TRUE;
 }
@@ -336,7 +484,6 @@ hildon_desktop_popup_window_visibility_notify (GtkWidget          *widget,
 {
   HildonDesktopPopupWindow *popup = HILDON_DESKTOP_POPUP_WINDOW (widget);
 
-  g_debug ("%s visibility_notify %p",__FILE__,popup);
   /* We have to close every window when called this but also we have to 
    * track if our composited windows are not the responsibles for the 
    * visibility-notify
@@ -398,6 +545,60 @@ hildon_desktop_popup_window_visibility_notify (GtkWidget          *widget,
   return FALSE;
 }
 
+static gboolean
+hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
+                                                       GdkEventButton *event,
+                                                       HildonDesktopPopupWindow *popup)
+{
+  gboolean in_panes_area  = FALSE,
+           in_window_area = FALSE;
+  gint x,y,w,h,i;
+
+  if (!event)
+    return FALSE;
+
+  gtk_widget_get_pointer (widget, &x, &y);
+
+  w = widget->allocation.width;
+  h = widget->allocation.height;
+
+  if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+    in_panes_area = TRUE;
+  else
+  {
+     w = GTK_WIDGET (popup)->allocation.width;
+     h = GTK_WIDGET (popup)->allocation.height;
+
+     gtk_widget_get_pointer (widget, &x, &y);
+
+     if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+       in_window_area = TRUE;
+
+     for (i=0; i < popup->priv->n_extra_panes; i++)
+     {
+       if (widget != popup->priv->extra_panes[i])
+       {
+         w = popup->priv->extra_panes[i]->allocation.width;
+         h = popup->priv->extra_panes[i]->allocation.height;
+
+         gtk_widget_get_pointer (popup->priv->extra_panes[i], &x, &y);
+
+         if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
+         {		 
+           in_panes_area = TRUE;
+           break;
+	 }
+       }
+     }
+  }
+
+  /* Event outside of popup or in button area, close in clean way */
+  if (!in_panes_area || in_window_area)
+    hildon_desktop_popup_window_popdown (popup);
+
+  return TRUE;
+}
+
 static gboolean
 hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
                                                   GdkEventButton *event)
@@ -415,7 +616,6 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
   w = widget->allocation.width;
   h = widget->allocation.height;
 
-  /* Pointer on window popup area */
   if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
     in_window_area = TRUE;
   else
@@ -427,7 +627,6 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
 
        gtk_widget_get_pointer (popup->priv->extra_panes[i], &x, &y);
 
-       /* Pointer on button area  */
        if ((x >= 0) && (x <= w) && (y >= 0) && (y <= h))
          in_panes_area = TRUE;
 
@@ -445,11 +644,13 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
 static void 
 hildon_desktop_popup_window_calculate_position (HildonDesktopPopupWindow *popup)
 {
-  gint x=0,y=0,i;
-  GtkRequisition req;
+  gint x=0,y=0,i,d_width=0;
+  GtkRequisition orig_req, req, req_pane;
 
   gtk_widget_size_request (GTK_WIDGET (popup), &req);
 
+  orig_req = req;
+
   if (popup->priv->position_func)
   {
     (* popup->priv->position_func) (popup, &x, &y, popup->priv->position_func_data);
@@ -461,15 +662,33 @@ hildon_desktop_popup_window_calculate_position (HildonDesktopPopupWindow *popup)
   {
     if (popup->priv->direction == HD_POPUP_WINDOW_DIRECTION_RIGHT_BOTTOM)
       for (i=0; i < popup->priv->n_extra_panes; i++)
+      {	
+	gtk_widget_size_request (popup->priv->extra_panes[i], &req_pane);
+
+	if (i > 0)
+          gtk_widget_size_request (popup->priv->extra_panes[i-1], &req);
+
+	d_width += req.width;
+	
         gtk_window_move (GTK_WINDOW (popup->priv->extra_panes[i]),
-			 req.width*(i+1) + x,
-			 y);
+			 d_width + x,
+			 y + orig_req.height - req_pane.height);
+      }
     else
     if (popup->priv->direction == HD_POPUP_WINDOW_DIRECTION_LEFT_TOP)	    
       for (i=0; i < popup->priv->n_extra_panes; i++)
+      {	      
+        gtk_widget_size_request (popup->priv->extra_panes[i], &req_pane);
+
+        if (i > 0)
+          gtk_widget_size_request (popup->priv->extra_panes[i-1], &req);
+
+        d_width -= req.width;
+
         gtk_window_move (GTK_WINDOW (popup->priv->extra_panes[i]),
-                         req.width*(i+1) - x,
-                         y);
+                         d_width - x,
+                         y + orig_req.height - req_pane.height);
+      }
   }	  
   else
   if (popup->priv->orientation == GTK_ORIENTATION_VERTICAL)
@@ -630,6 +849,67 @@ hildon_desktop_popup_window_get_pane (HildonDesktopPopupWindow *popup, gint pane
     return popup->priv->extra_panes [pane];
 }	
 
+GtkWidget *
+hildon_desktop_popup_window_get_grabbed_pane (HildonDesktopPopupWindow *popup)
+{
+  g_assert (HILDON_DESKTOP_IS_POPUP_WINDOW (popup));
+
+  if (popup->priv->pane_with_grab)
+    return popup->priv->pane_with_grab;
+  else
+    return GTK_WIDGET (popup->priv->pane_with_grab);	   
+}	
+
+void
+hildon_desktop_popup_window_jump_to_pane (HildonDesktopPopupWindow *popup, gint pane)
+{
+  g_assert (HILDON_DESKTOP_IS_POPUP_WINDOW (popup));
+
+  if (pane >= popup->priv->n_extra_panes)
+    pane = popup->priv->n_extra_panes -1;	  
+	
+  if (GTK_WIDGET_VISIBLE (GTK_WIDGET (popup)))
+  {
+    if (pane <= -1)
+    {
+      if (popup->priv->have_xgrab)	    
+        return;
+      else
+      {
+        gtk_grab_remove (popup->priv->pane_with_grab);	      
+
+	popup_grab_on_window (GTK_WIDGET (popup)->window, GDK_CURRENT_TIME, TRUE);
+
+        gtk_grab_add (GTK_WIDGET (popup));
+
+        popup->priv->pane_with_grab = NULL;
+        popup->priv->have_xgrab = TRUE;
+      }
+    }
+    else
+    {
+      GtkWidget *widget_with_grab;
+	    
+      if (popup->priv->have_xgrab)
+        widget_with_grab = GTK_WIDGET (popup);
+      else
+        widget_with_grab = popup->priv->pane_with_grab;	      
+      	
+      if (widget_with_grab == popup->priv->extra_panes[pane])
+        return;
+      
+      gtk_grab_remove (widget_with_grab);
+
+      popup_grab_on_window (popup->priv->extra_panes[pane]->window, GDK_CURRENT_TIME, TRUE);
+
+      gtk_grab_add (popup->priv->extra_panes[pane]);
+
+      popup->priv->pane_with_grab = popup->priv->extra_panes[pane];
+      popup->priv->have_xgrab = FALSE;	
+    }	    
+  }
+}	
+
 void 
 hildon_desktop_popup_window_popup (HildonDesktopPopupWindow *popup,
 				   HDPopupWindowPositionFunc func,
@@ -667,18 +947,25 @@ hildon_desktop_popup_window_popup (HildonDesktopPopupWindow *popup,
 
   hildon_desktop_popup_window_calculate_position (popup);
 
-  gtk_widget_show (GTK_WIDGET (popup));
+  gtk_widget_show_all (GTK_WIDGET (popup));
 
   popup_grab_on_window (GTK_WIDGET (popup)->window, activate_time, TRUE); /* Should always succeed */
 
   gtk_grab_add (GTK_WIDGET (popup));
+
+  g_signal_emit_by_name (popup, "popup-window");
 }
 
 void 
 hildon_desktop_popup_window_popdown (HildonDesktopPopupWindow *popup)
 {
+  gint i;
+	
   gtk_widget_hide (GTK_WIDGET (popup));
 
   gtk_grab_remove (GTK_WIDGET (popup));
+
+  for (i=0; i < popup->priv->n_extra_panes; i++)
+    gtk_grab_remove (popup->priv->extra_panes[i]);	  
 }
 
