@@ -36,7 +36,9 @@
 #include <gtk/gtkmain.h>
 
 #ifdef HAVE_LIBHILDON
+#define ENABLE_UNSTABLE_API
 #include <hildon/hildon-helper.h>
+#undef ENABLE_UNSTABLE_API
 #else
 #include <hildon-widgets/hildon-defines.h>
 #endif
