@@ -54,6 +54,9 @@ typedef enum HDKeyAction
   HD_KEY_ACTION_FULLSCREEN,
   HD_KEY_ACTION_ZOOM_IN,
   HD_KEY_ACTION_ZOOM_OUT,
+  HD_KEY_ACTION_SWITCH_WINDOW_PREV,
+  HD_KEY_ACTION_SWITCH_WINDOW_NEXT,
+  HD_KEY_ACTION_APPLICATION
 }
 HDKeyAction;
 
