@@ -286,6 +286,15 @@ void
 hd_wm_activate_service (const gchar *service_name,
                         const gchar *launch_parameters);
 
+void 
+hd_wm_switch_application_window (HDWM *hdwm, HDWMWatchableApp *app, gboolean to_next);
+
+void 
+hd_wm_switch_info_window (HDWM *hdwm, HDEntryInfo *entry_info, gboolean to_next);
+
+void 
+hd_wm_switch_instance_current_window (HDWM *hdwm, gboolean to_next);
+
 /*
  * These are simple getters/setters that replace direct use of global
  * hnwm->something. In order to ensure that we do not incure performance
@@ -293,7 +302,6 @@ hd_wm_activate_service (const gchar *service_name,
  * 'extern inline'; this ensures that these functions will always be inlined.
  */
 
-
 extern inline Atom
 hd_wm_get_atom(gint indx);
 
